#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <sstream>
#include <math.h>
#include <sys/time.h>
#include "System.h"
#include "Utility.h"
#include "SNPData.h"
#include "Segment.h"
#include "Simulate.h"
#include "RelationshipUtils.h"

int createPhaseErrors(SNPData &data, int idx, double switchProb,
		      vector<int> select, Simulate::Diplo &diplo, 
		      Simulate::Diplo &newDiplo, vector<int> &switchPos){
  switchPos.clear();
  newDiplo.pop = diplo.pop;
  newDiplo.gender = diplo.gender;
  newDiplo.idx = diplo.idx;
  newDiplo.haplo_sry[0] = diplo.haplo_sry[0];
  newDiplo.haplo_sry[1] = diplo.haplo_sry[1];

  int haploIdx0 = data.diplos[idx].haplos[0];
  int haploIdx1 = data.diplos[idx].haplos[1];

  int numHeter = 0;
  for(int pos = 0; pos < data.seq_len; ++pos){
    if(getAllele(data, haploIdx0, pos) != getAllele(data, haploIdx1, pos)){
      numHeter++;
      if(select[pos] == 1 && drand() < switchProb) 
	switchPos.push_back(pos);
    }
  }
  switchPos.push_back(data.seq_len);

  vector< vector< pair<Segment, int> > > newSources(2);
  bool sw = 0;
  vector<int> segmentIdx(2, 0);
  for(int pos = 0; pos < switchPos.size(); ++pos){
    for(int i = 0; i <= 1; ++i){
      while(segmentIdx[i ^ sw] < diplo.haplos[i ^ sw].size() && 
	    diplo.haplos[i ^ sw].sources[segmentIdx[i ^ sw]].first.right() < switchPos[pos]){
	newSources[i].push_back(diplo.haplos[i ^ sw].sources[segmentIdx[i ^ sw]]);
	if(pos > 0 && newSources[i].back().first.left() < switchPos[pos - 1])
	  newSources[i].back().first.SetLeft(switchPos[pos - 1]);
	segmentIdx[i ^ sw]++;
      }    
      if(segmentIdx[i ^ sw] < diplo.haplos[i ^ sw].size()){
	newSources[i].push_back(diplo.haplos[i ^ sw].sources[segmentIdx[i ^ sw]]);
	int segmentStart = pos > 0?switchPos[pos - 1]:0;
	if(newSources[i].size() > 1){
	  segmentStart = max(segmentStart, newSources[i][newSources[i].size() - 2].first.right());
	}
	newSources[i].back().first.SetLeft(segmentStart);
	newSources[i].back().first.SetRight(switchPos[pos]);
      }
    }
    sw = 1 - sw;
  }
  Simulate::Haplo newHaplos[2];
  newHaplos[0].sources = newSources[0];
  newHaplos[1].sources = newSources[1];
  
  newDiplo.haplos[0] = newHaplos[0];
  newDiplo.haplos[1] = newHaplos[1];
  
  return numHeter;
}

int Main(vector<string> args){
  /*
  AssertMsg(args.size() == 4,
	    "Usage: CreatePhaseErrorsMain binFile initialPairFile pairFile");

  string binPref = args[1], initialPairPref = args[2], pairPref = args[3];

  stringstream stream;
  stream << binPref << "chrom1.snpprofiledata.bin";
  SNPProfile profile, prunedProfile;
  SNPData data, prunedData;
  ReadBinSNPProfileData(stream.str(), profile, data);

  stream.clear();
  stream.str("");
  stream << initialPairPref << "chrom1";
  ifstream initialPairFile(stream.str().c_str());
  Simulate::Diplo indivA, indivB;
  int idxA = restoreIndividual(data, data, indivA, initialPairFile);
  int idxB = restoreIndividual(data, data, indivB, initialPairFile);
  initialPairFile.close();
  printDiploSegments(indivA);

  double err = 0.0;
  for(int i = 0; i < data.seq_len; ++i){
    err = max(err, (double)SNPData::switch_decode(data.diplos[idxA].switches[i]));
  }
  cerr << "Phasing error for B: " << err << endl;

  vector<int> switchesA, switchesB;
  stream.clear();
  stream.str("");
  stream << pairPref << "chrom1_switches";
  ifstream switchFile(stream.str().c_str());
  ReadBin(switchFile, switchesA);
  ReadBin(switchFile, switchesB);
  switchFile.close();
  
  cerr << "First switches: ";
  int switchesToPrint = switchesB.size() < 50? switchesA.size(): 50;
  for(int i = 0; i < switchesToPrint; ++i){
    cerr << switchesA[i] << " ";
  }
  cerr << endl;

  stream.clear();
  stream.str("");
  stream << pairPref << "chrom1";
  ifstream pairFile(stream.str().c_str());
  Simulate::Diplo newIndivA, newIndivB;
  int newIdxA = restoreIndividual(data, data, newIndivA, pairFile);
  int newIdxB = restoreIndividual(data, data, newIndivB, pairFile);
  pairFile.close();
  cerr << "After errors" << endl;
  printDiploSegments(newIndivA);

  err = 0.0;
  for(int i = 0; i < data.seq_len; ++i){
    err = max(err, (double)SNPData::switch_decode(data.diplos[newIdxA].switches[i]));
  }
  cerr << "Phasing error for A: " << err << endl;

  data.Clear();

  stream.clear();
  stream.str("");
  stream << binPref << "Pruned_chrom1.snpprofiledata.bin";
  ReadBinSNPProfileData(stream.str(), prunedProfile, prunedData);
  stream.clear();
  stream.str("");
  stream << pairPref << "chrom1_pruned";
  Simulate::Diplo newPrunedIndivA, newPrunedIndivB;
  ifstream prunedPairFile(stream.str().c_str());
  newIdxA = restoreIndividual(prunedData, prunedData, newPrunedIndivA, prunedPairFile);
  newIdxB = restoreIndividual(prunedData, prunedData, newPrunedIndivB, prunedPairFile);
  prunedPairFile.close();
  cerr << "Pruned" << endl;
  printDiploSegments(newPrunedIndivA);

  err = 0.0;
  for(int i = 0; i < prunedData.seq_len; ++i){
    err = max(err, (double)SNPData::switch_decode(prunedData.diplos[newIdxA].switches[i]));
  }
  cerr << "Phasing error for A: " << err << endl;
  */
  
  string usage = "Usage: CreatePhaseErrorsMain binPref pairPref outPref switchProb";
  AssertMsg(args.size() == 5, usage);
  
  string binPref = args[1], pairPref = args[2], outPref = args[3];
  double switchProb = 0.0;
  stringstream stream;
  stream.str(args[4]);
  stream >> switchProb;  
 
  for(int chr = 1; chr <= 22; ++chr){
    //Read the full SNP profile (from which the individuals were originally made)
    stream.clear();
    stream.str("");
    stream << binPref << "Cleared_chrom" << chr << ".snpprofiledata.bin";
    SNPProfile profile;
    SNPData data;
    ReadBinSNPProfileData(stream.str(), profile, data);
    cerr << "Chr" << chr << ": ";

    //Read the full individuals
    stream.clear();
    stream.str("");
    stream << pairPref << "chrom" << chr << "_cleared";
    ifstream pairFile(stream.str().c_str());
    Simulate::Diplo indivA, indivB;
    vector<int> relIndices(2, 0);
    relIndices[0] = restoreIndividual(data, data, indivA, pairFile);
    relIndices[1] = restoreIndividual(data, data, indivB, pairFile);
    pairFile.close();
    string labelA = data.diplos[relIndices[0]].label;
    string labelB = data.diplos[relIndices[1]].label;
    //printDiploSegments(indivA);

    //The select vector will be a vector of
    //0's and 1's that specifies which SNPs of the full data 
    //are in the pruned data.
    stream.clear();
    stream.str("");
    stream << binPref << "Pruned_Cleared_chrom" << chr << "_select";
    vector<int> select;
    ifstream selectFile(stream.str().c_str());
    ReadBin(selectFile, select);
    
    //Introduce phasing errors, but only on SNPs that are heterozygous
    //AND present in the pruned data.
    Simulate::Diplo newIndivA, newIndivB;
    vector<int> switchesA, switchesB;
    int numHeterA = createPhaseErrors(data, relIndices[0], switchProb, 
				      select, indivA, newIndivA, switchesA);
    int numHeterB = createPhaseErrors(data, relIndices[1], switchProb, 
				      select, indivB, newIndivB, switchesB);
    cerr << switchesA.size() << " switches in A ("
	 << numHeterA << " heter pos) and " << switchesB.size()
	 << " in B (" << numHeterB << " heter pos)" << endl;
    //printDiploSegments(newIndivA);

    stream.clear();
    stream.str("");
    stream << outPref << "chrom" << chr << "_switches";
    ofstream switchFile(stream.str().c_str());
    WriteBin(switchFile, switchesA);
    WriteBin(switchFile, switchesB);
    switchFile.close();

    //Store the full individuals with the phasing errors
    stream.clear();
    stream.str("");
    stream << outPref << "chrom" << chr << "_cleared";
    ofstream outfile1(stream.str().c_str());
    storeIndividual(newIndivA, switchProb, labelA, outfile1);
    storeIndividual(newIndivB, switchProb, labelB, outfile1);    
    outfile1.close();

    //Restrict the individuals with the phasing errors to
    //the pruned data.
    vector<int> resultMap;
    newIndivA.RestrictDiplo(select, resultMap);
    newIndivB.RestrictDiplo(select, resultMap);
    stream.clear();
    stream.str("");
    stream << outPref << "chrom" << chr << "_pruned";
    ofstream outfile2(stream.str().c_str());
    storeIndividual(newIndivA, switchProb, labelA, outfile2);
    storeIndividual(newIndivB, switchProb, labelB, outfile2);    
    outfile2.close();
    //printDiploSegments(newIndivA);
  }
  
  /*
  string binPath = args[1], pairPath = args[2], selectPath = args[3];
  ifstream pairFile(pairPath.c_str());
  //stringstream input(args[7]);
  double switchProb = 0.02;

  //timeval tv;
  //double start, end;

  //gettimeofday(&tv, NULL);
  //start = tv.tv_sec + (tv.tv_usec/1000000.0);  
  //Read SNPProfile and SNPData.
  SNPProfile profile;
  SNPData data;
  ReadBinSNPProfileData(binPath, profile, data);

  //Read stored individuals
  Simulate::Diplo indivA, indivB;
  vector<int> relIndices(2, 0);
  relIndices[0] = restoreIndividual(data, data, indivA, pairFile);
  relIndices[1] = restoreIndividual(data, data, indivB, pairFile);
  pairFile.close();

  int haploIdxA0 = data.diplos[relIndices[0]].haplos[0];
  int haploIdxA1 = data.diplos[relIndices[0]].haplos[1];
  cerr << "Haplotypes of A " << haploIdxA0 << " and " << haploIdxA1 << endl;

  vector<int> switchPos;
  int numHeter = 0;
  for(int pos = 0; pos < data.seq_len; ++pos){
    if(getAllele(data, haploIdxA0, pos) != getAllele(data, haploIdxA1, pos)){
      //cerr << (int)getAllele(data, haploIdxA0, pos) << ", " << (int)getAllele(data, haploIdxA1, pos) << endl;
      numHeter++;
      if(drand() < 0.02) 
	switchPos.push_back(pos);
    }
  }
  switchPos.push_back(data.seq_len);

  vector< vector< pair<Segment, int> > > newSources(2);
  bool sw = 0;
  vector<int> segmentIdx(2, 0);
  for(int pos = 0; pos < switchPos.size(); ++pos){
    for(int i = 0; i <= 1; ++i){
      while(segmentIdx[i ^ sw] < indivA.haplos[i ^ sw].size() && 
	    indivA.haplos[i ^ sw].sources[segmentIdx[i ^ sw]].first.right() < switchPos[pos]){
	newSources[i].push_back(indivA.haplos[i ^ sw].sources[segmentIdx[i ^ sw]]);
	if(pos > 0 && newSources[i].back().first.left() < switchPos[pos - 1])
	  newSources[i].back().first.SetLeft(switchPos[pos - 1]);
	segmentIdx[i ^ sw]++;
      }    
      if(segmentIdx[i ^ sw] < indivA.haplos[i ^ sw].size()){
	newSources[i].push_back(indivA.haplos[i ^ sw].sources[segmentIdx[i ^ sw]]);
	int segmentStart = pos > 0?switchPos[pos - 1]:0;
	if(newSources[i].size() > 1){
	  segmentStart = max(segmentStart, newSources[i][newSources[i].size() - 2].first.right());
	}
	newSources[i].back().first.SetLeft(segmentStart);
	newSources[i].back().first.SetRight(switchPos[pos]);
      }
    }
    sw = 1 - sw;
  }
  cerr << "#total = " << data.seq_len << ", #heterozygous = "
       << numHeter << ", #switches = " << switchPos.size() << endl; 
  printDiploSegments(indivA);
  cerr << "Switches ";
  for(int i = 0; i < switchPos.size(); ++i)
    cerr << switchPos[i] << " ";
  cerr << endl;
  Simulate::Haplo newHaplos[2];
  newHaplos[0].sources = newSources[0];
  newHaplos[1].sources = newSources[1];
  Simulate::Diplo newDiplo;
  newDiplo.haplos[0] = newHaplos[0];
  newDiplo.haplos[1] = newHaplos[1];
  printDiploSegments(newDiplo);

  vector<int> select;
  ifstream selectFile(selectPath.c_str());
  Read(selectFile, select);
  vector<int> resultMap;
  newDiplo.RestrictDiplo(select, resultMap);
  printDiploSegments(newDiplo);
  for(int i = 0; i < 1000;++i){
    cerr << select[i] << " ";
    if(i%10 == 0) cerr << endl;
  }
  cerr << endl;
  */
}
