#include "RelationshipModel.h"

RelationshipModel::RelationshipModel(SNPData &inData, int inIdxA, int inIdxB,
				     vector< vector<double> > &inLetterFreq,
				     vector< vector<double> > &inErrorFunction){
  //cerr << "In RelationshipModel::RelationshipModel(...)" << endl;
  init(inData, inIdxA, inIdxB, inLetterFreq, inErrorFunction);
}

void RelationshipModel::init(SNPData &inData, int inIdxA, int inIdxB,
			     vector< vector<double> > &inLetterFreq,
			     vector< vector<double> > &inErrorFunction){
  //cerr << "In RelationshipModel::init(...)" << endl;
  data = &inData;
  AssertMsg(inIdxA >= 0 && inIdxA < (*data).num_diplos, "Invalid index for individual A.");
  idxA = inIdxA;
  AssertMsg(inIdxB >= 0 && inIdxB < (*data).num_diplos, "Invalid index for individual B.");
  idxB = inIdxB;
  AssertMsg(inLetterFreq.size() == (*data).seq_len, 
	    "The matrix of letter frequencies and the data object have incompatible sizes."); 
  letterFreq = &inLetterFreq;
  errorFunction = &inErrorFunction;
}

void RelationshipModel::init(SNPData *inData, int inIdxA, int inIdxB,
			     vector< vector<double> > *inLetterFreq,
			     vector< vector<double> > *inErrorFunction){
  init(*inData, inIdxA, inIdxB, *inLetterFreq, *inErrorFunction);
}

/* 
 * The copy constructor will create copies of the pointers.
 * This is done on purpose, to avoid copying "data" and "letterFreq."
 * This is safe as long as the members of the class respect the rule that
 * they should not delete any pointers.
 */
RelationshipModel::RelationshipModel(const RelationshipModel &copy){
  init(copy.data, copy.idxA, copy.idxB, copy.letterFreq, copy.errorFunction);
}

RelationshipModel& RelationshipModel::operator=(const RelationshipModel &that){
  if(this != &that)
    init(that.data, that.idxA, that.idxB, that.letterFreq, that.errorFunction);
  return *this;
}
