#ifndef RELATIONSHIPUTILS_H
#define RELATIONSHIPUTILS_H

#include <stdio.h>
#include <stdlib.h>
#include "System.h"
#include "Utility.h"
#include "SNPData.h"
#include "Simulate.h"

#define INF std::numeric_limits<double>::max()
#define NINF -INF
#define ZERO std::numeric_limits<double>::min() 

vector<int> createRelatives(SNPProfile &profile, SNPData &data, 
			    Simulate::Diplo &indivA, Simulate::Diplo &indivB,
			    int numMRCAs, int genA, int genB, int maxIndiv, 
			    bool verbose = false, bool addToData = true);
vector<int> createUnrelated(SNPProfile &profile, SNPData &data, 
			    Simulate::Diplo &indivA, Simulate::Diplo &indivB, int maxIndiv, bool verbose);
void printSegment(pair<Segment, int> &s);
void printDiploSegments(Simulate::Diplo &diplo);
void printDiploSegments(SNPData &data, int idx);
string getChildLabel(string mate0Label, string mate1Label, int childNum);
Simulate::Diplo getSimulateDiploAt(SNPData &data, int idx);
void setSimulateHaploSources(const vector<int8> haploData, Simulate::Haplo &haplo);
void mate(SNPProfile &profile, SNPData &data, Simulate::Diplo &mate0,
		  Simulate::Diplo &mate1,Simulate::Diplo &child, string childLabel, bool verbose, bool add);
void trueIBDCounts(Simulate::Diplo &diplo1, Simulate::Diplo &diplo2, 
		   int counts[2], vector< vector<int> > &indicators);
vector<int> trueIBDBlocks(Simulate::Diplo &diplo1, Simulate::Diplo &diplo2, vector<int> & NIT_CausedBy, vector<int> & INT_CausedBy, int & NIT_tolTransitions, int & INT_tolTransitions);
double computeLogSum(vector<double> logs);
double computeLogSumOfTwo(double log1, double log2);
double correctLog(double badNum);
int getAllele(SNPData& data, int haploIdx, int pos);

void initErrorFunction(vector< vector<double> > &errorFunction);
void computeLetterFreq(SNPData &data, double count, 
		       vector< vector<double> > &letterFreq,
		       int size = 5);
void computePascalTri(vector< vector<int> > &pascalTri, int max);
void computeRecomb(SNPData &data, 
		   vector< vector< double> > &logRecomb,
		   vector< vector< double> > &logNoRecomb, 
		   int maxGens);
double squareErrorIBD(vector<double> &predictions, vector<bool> &trueIBD);
bool areAllMissing(SNPData &data, int snpIdx);
void storeIndividual(Simulate::Diplo &diplo, double switchProb, string label,
		     ofstream &outfile);
int restoreIndividual(SNPData &data, Simulate::Diplo &diplo, ifstream &infile);
int restoreIndividual(SNPData &data, SNPData &addToData,
		      Simulate::Diplo &diplo, ifstream &infile);
void restoreIndividual(Simulate::Diplo &diplo, ifstream &infile,double &switchProb, int &pop, string &label);

template <typename T>
void printGenericMatrix(vector< vector<T> > &matrix, 
			ostream &os = cout, int lines = -1){
  int linesToPrint = (lines > 0 && lines < matrix.size())? lines: matrix.size();
  for(int i = 0; i < linesToPrint; ++i){
    for(int j = 0; j < matrix[i].size(); ++j){
      os << matrix[i][j];
      if(j < matrix[i].size() - 1) os << "\t";
    }
    os << endl;
 }
}
#endif
