#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <sstream>
#include <math.h>
#include "System.h"
#include "Utility.h"
#include "SNPData.h"
#include "Simulate.h"
#include "RelationshipModel.h"
#include "RelationshipUtils.h"

int Main(vector<string> args){  
  string profilePath = "../hapmap/hapmap550_1_in_250.snpprofiledata.bin";
  SNPProfile profile;
  SNPData data;
  ReadBinSNPProfileData(profilePath, profile, data);

  vector< vector<double> > errorFunction;
  initErrorFunction(errorFunction);
  vector< vector<double> > letterFreq;
  computeLetterFreq(data, 0.01, letterFreq);
  vector< vector<int> > pascalTri;
  computePascalTri(pascalTri, 3);
  vector< vector<double> > logRecomb, logNoRecomb;
  computeRecomb(data, logRecomb, logNoRecomb, 3);

  int numInitialIndiv = data.num_diplos;
  Simulate::Diplo indivA, indivB;
  vector<int> relIndices = createRelatives(profile, data, indivA, indivB, 
					   2, 0, 1, numInitialIndiv, false, true);
  cerr << "Created individuals " << relIndices[0] << " " << relIndices[1] << endl;
  /*
  TwoMRCAModel model1(data, relIndices[0], relIndices[1], 
		      letterFreq, errorFunction, pascalTri,
		      logRecomb, logNoRecomb, 1, 1);
  cerr << "Created model" << endl;
  vector< vector<double> > forwardProb;
  double seqProb1 = model1.forward(forwardProb);
  cerr << "Probability for A and B = " << seqProb1 << endl;
  */
  string filename = "test.diplo.bin";
  ofstream outfile(filename.c_str());
  storeIndividual(indivA, 0.0, "indivA", outfile);
  storeIndividual(indivB, 0.0, "indivB", outfile);
  outfile.close();
  cerr << "Stored individuals..." << endl;

  ifstream infile(filename.c_str());
  Simulate::Diplo restoredA, restoredB;
  restoreIndividual(data, restoredA, infile);
  restoreIndividual(data, restoredB, infile);
  infile.close();
  cerr << "Restored individuals..." << endl;
  /*
  TwoMRCAModel model2(data, data.num_diplos - 2, data.num_diplos - 1, 
		      letterFreq, errorFunction, pascalTri,
		      logRecomb, logNoRecomb, 1, 1);
  double seqProb2 = model2.forward(forwardProb);
  cerr << "Probability for restored A and B = " << seqProb2 << endl;  
  */
}

int Main2(vector<string> args){  
  string profilePath = "../../hapmap/hapmap550.snpprofiledata.bin";
  SNPProfile profile;
  SNPData data;
  ReadBinSNPProfileData(profilePath, profile, data);

  vector< vector<double> > errorFunction;
  initErrorFunction(errorFunction);
  vector< vector<double> > letterFreq;
  computeLetterFreq(data, 0.01, letterFreq);
  vector< vector<int> > pascalTri;
  computePascalTri(pascalTri, 3);
  vector< vector<double> > logRecomb, logNoRecomb;
  computeRecomb(data, logRecomb, logNoRecomb, 3);

  int numInitialIndiv = data.num_diplos;
  Simulate::Diplo indivA, indivB;
  vector<int> relIndices = createRelatives(profile, data, indivA, indivB, 
					   2, 0, 0, numInitialIndiv, true, true);
   
  string filename = "test.diplo.bin";
  ofstream outfile(filename.c_str());
  indivA.WriteBin(outfile);
  /*
  SNPData::Diplo indivA2 = data.diplos[relIndices[0]];
  indivA2.WriteBin(outfile);
  */
  Write(outfile, 0.0);
  Write(outfile, data.diplos[relIndices[0]].pop);
  Write(outfile, data.diplos[relIndices[0]].label);
 
  /*
  int haploIdx0 = data.diplos[relIndices[0]].haplos[0];
  vector<int8> haplos0 = data.haplos[haploIdx0].data;
  WriteBin(outfile, haplos0);
  int haploIdx1 = data.diplos[relIndices[0]].haplos[1];
  vector<int8> haplos1 = data.haplos[haploIdx1].data;
  WriteBin(outfile, haplos1);
  */
  outfile.close();

  ifstream infile(filename.c_str());
  Simulate::Diplo indivC;
  indivC.ReadBin(infile);
  indivC.idx = data.num_diplos - 1;
  /*
  SNPData::Diplo indivC2;
  indivC2.ReadBin(infile);
  */
  double switchProb;
  Read(infile, switchProb);
  int pop;
  Read(infile, pop);
  string label;
  Read(infile, label);
  /*
  vector<int8> haplos0read, haplos1read;
  ReadBin(infile, haplos0read);
  ReadBin(infile, haplos1read);
  */
  infile.close();
  
  cerr << "Read individual " << label 
       << ", pop = " << pop
       << ", switch = " << switchProb << endl;
  
  printDiploSegments(indivC);
  
  vector < vector < int8 > > childHaplos(2);
  cerr << "Created vectors" << endl;
  indivC.Generate(data, childHaplos[0], childHaplos[1]);
  cerr << "Generated haplos" << endl;
  data.AddDiplo(childHaplos[0], childHaplos[1], switchProb, pop, label);
  cerr << "Added diplo" << endl;
  data.ComputeDerived();
  cerr << "Computed derived" << endl;
  
  /*
  data.AddDiplo(haplos0read, haplos1read, 0.0, 
		0, "");
  data.ComputeDerived();
  */
  for(int i = 0; i < data.diplos[data.num_diplos - 1].switches.size(); ++i){
    Assert(data.diplos[data.num_diplos - 1].switches[i] == 
	   data.diplos[relIndices[0]].switches[i]);
  }
  TwoMRCAModel model1(data, relIndices[0], relIndices[1], 
		      letterFreq, errorFunction, pascalTri,
		      logRecomb, logNoRecomb, 0, 0);
  vector< vector<double> > forwardProb;
  double seqProb1 = model1.forward(forwardProb);

  TwoMRCAModel model2(data, data.num_diplos - 1, relIndices[1], 
		      letterFreq, errorFunction, pascalTri, 
		      logRecomb, logNoRecomb, 0, 0);
  double seqProb2 = model2.forward(forwardProb);
  cerr << "Probability for A and B = " << seqProb1 << endl;
  cerr << "Probability for C and B = " << seqProb2 << endl;
}
