#include "RelationshipModel.h"

UnrelatedModel::UnrelatedModel(SNPData &inData, int inIdxA, int inIdxB,
			       vector< vector<double> > &inLetterFreq,
			       vector< vector<double> > &inErrorFunction){
  RelationshipModel::init(inData, inIdxA, inIdxB, inLetterFreq, inErrorFunction);
}

/*
 * Return value:
 * The log-likelihood of the sequences for the two individuals under
 * consideration, under the assumption that they are unrelated.
 * Smallest value returned is log(ZERO).
 */
double UnrelatedModel::logLikelihood(){
  //Indices in the haplos vector of the SNPData object.
  int haploAIdx = 2 * idxA;
  int haploBIdx = 2 * idxB;

  //Assume that if the two individuals are unrelated, then all
  //positions are independent (from each other and between the 
  //two individuals).
  double logProb = 0.0;
  for(int pos = 0; pos < (*data).seq_len; ++pos){
    double probA1 = 0.0, probA2 = 0.0, probB1 = 0.0, probB2 = 0.0;
    for(int letter = 0; letter < (*errorFunction).size(); ++letter){
      probA1 += (*letterFreq)[pos][letter] * 
	(*errorFunction)[letter][(int)getAllele(*data, haploAIdx, pos)];
      probA2 += (*letterFreq)[pos][letter] *
	(*errorFunction)[letter][(int)getAllele(*data, haploAIdx + 1, pos)];
      probB1 += (*letterFreq)[pos][letter] * 
	(*errorFunction)[letter][(int)getAllele(*data, haploBIdx, pos)];
      probB2 += (*letterFreq)[pos][letter] *
	(*errorFunction)[letter][(int)getAllele(*data, haploBIdx + 1, pos)];
    }
    logProb += log(probA1) + log(probA2) + log(probB1) + log(probB2);
  }
  return logProb;
} 
