% Read the rels in classifyRels from the specified files and classify them
% by degree.
function degreeData = classifyDegree(filenames, numFolds, classifyRels, numIndivPerSet)

% Get the relationships whose features we use to classify the degree.
degreeRels = degreeClassificationRels();
degreeData = loadClassifierData(filenames, classifyRels, degreeRels, numIndivPerSet);
classifyRelIdx = 1:length(classifyRels);

numCounts = 4; % haplocount features
numBlockStats = 1; % block statistics
numCols = numCounts * length(degreeRels) + numBlockStats * length(degreeRels) + length(degreeRels);
blockStartIdx = numCounts * length(degreeRels) + 1;
likeStartIdx = blockStartIdx + numBlockStats * length(degreeRels);

unrelIdx = 0;
for r = 1:length(degreeRels),
    if strcmp(degreeRels{r}, 'unrel')
        unrelIdx = r;
        break;
    end
end
if unrelIdx == 0,
    warning('', 'unrel not found, using max likelihood to normalize')
end

relAssign = getRelDegree(classifyRels);

haploAidx = 2:4:(blockStartIdx - 1);
haploBidx = 3:4:(blockStartIdx - 1);
for r = 1: size(degreeData, 1),
    for i = 1:size(degreeData, 2),
        if unrelIdx > 0,
            norm = degreeData(r, i, likeStartIdx + unrelIdx - 1);
        else
            norm = max(degreeData(r, i, likeStartIdx:numCols));
        end
        degreeData(r, i, likeStartIdx:numCols) = degreeData(r, i, likeStartIdx:numCols) - norm;
        % Replace 0 haplocounts with 0.5 and normalize them.
        for d = 1:length(haploAidx),
            if degreeData(r, i, haploAidx(d)) == 0 && degreeData(r, i, haploBidx(d)) == 0,
                degreeData(r, i, haploAidx(d)) = 0.5;
                degreeData(r, i, haploBidx(d)) = 0.5;
            end
        end
        haploNorm = degreeData(r, i, haploAidx) + degreeData(r, i, haploBidx);
        degreeData(r, i, haploAidx) = degreeData(r, i, haploAidx) ./ haploNorm;
        degreeData(r, i, haploBidx) = degreeData(r, i, haploBidx) ./ haploNorm;

        %degreeData(r, i, likeStartIdx + 3) = mean(degreeData(r, i, (likeStartIdx + 3):(likeStartIdx + 4)));
        %degreeData(r, i, blockStartIdx + 3) = mean(degreeData(r, i, (blockStartIdx + 3):(blockStartIdx + 4)));
    end
end

if unrelIdx > 0
    selLikelihoods = [1, 2, 3, 4, 5, 6, 7, 8];
else
   selLikelihoods = 1:length(degreeRels);
end
% Do NOT include the IBD percentage under PO, it is ALWAYS 0.5.
selBlocks = [1, 4, 6, 7, 8];
selPostcounts = [];
selCols = [];
for s = selPostcounts,
    % get only the num of transitions and not the rest of the postcounts
    selCols = [selCols, (s - 1) * numCounts + [2, 4]];
end
for s = selBlocks,
    selCols = [selCols, (s - 1) * numBlockStats + blockStartIdx - 1 + [1:numBlockStats]];
end
selCols = [selCols, likeStartIdx - 1 + selLikelihoods]
classifyPost2(degreeData, numFolds, classifyRels, degreeRels, classifyRelIdx, relAssign, selCols);
end