% Sums up the likelihoods for all chromosomes and for several relationships
% into a single file with one line of data.

% Individuals
minIndiv = 1;
maxIndiv = 100;

% Do we have regular pairs (0) or pairs from a tree (1)? This affects the naming
% conventions for accessing the files.
doTree = 0;
doLevels = 0;

% Set
set = 1;
pairPref = 'CEU_TSI';
setPref = strcat(pairPref, '_Set_', int2str(set));

% Directory with likelihoods (eg. Likelihoods/hapmap3_CEU_TSI)
parDir = '..';
likePref = strcat(parDir, '/LogProbs/hapmap3_', pairPref, '_phaseErr008/');

% Suffixes of input logprob files and suffix of output file.
% For each individual, and each chromosome, we will read all the 
% logProb files with suffixes in suf. Likelihoods for all chromosomes and
% for the same assumed relationship
% will be added. Then a single logProb file will be produced. The output
% will have the a single line of data with the sum of likelihoods for all 
% chromosomes (sum in log-space,
% product in non-log-space). The relationships for which the likelihoods
% will be computed (i.e. the assumed relationships in the output file)
% are in outRels. Each of the relationships in outRels
% should be in at least one of the input logProb files. If a relationship
% is in more than one input file, the last likelihood read for that
% relationship will overwrite all previously read values.
inDegree = 3;
outDegree = 3;

suf = {'_5', '_7'};
rels1d = {'2_0_0', '1_n1_0'};
rels2d = {'2_0_1', '1_0_0', '1_n1_1'};
rels3d = {'2_0_2', '2_1_1', '1_n1_2', '1_0_1'};
rels4d = {'2_1_2', '2_0_3', '1_n1_3', '1_0_2', '1_1_1'};
rels5d = {'2_2_2', '2_0_4', '2_1_3', '1_n1_4', '1_1_2', '1_0_3'};

switch inDegree,
    case 1
        inRels = rels1d;
    case 2
        inRels = rels2d;
    case 3
        inRels = rels3d;
    case 4
        inRels = rels4d;
    case 5
        inRels = rels5d;
    case 0
        inRels = {'unrel'};
    case 6
        inRels = {'distant'};
end

outRels = getDegreeAssumedRels(outDegree);
if outDegree > 0,
    outSuf = strcat('_allChr_pruned_', int2str(outDegree), 'd');
    if outDegree ~= 2,
        suf = [suf, strcat('_', int2str(outDegree), 'd')];
    else
        suf = [suf, {'_2d', '_2d'}];
    end
else
    outSuf = '_allChr_pruned_9';
    suf = [suf, '_9'];
end

% needed if we're using trees instead of pairs.
%inRels = ;

% The output file will contain the likelihoods for the following
% relationships. 
%outRels = rels4;
numCols = length(outRels);

% Tree-pairs. Not used if we're computing the likelihoods for "regular"
% pairs and not tree pairs.
levelPairs = [2, 3];%[1, 2; 1, 3; 1, 3; 2, 2; 2, 3; 2, 3; 2, 3; 3, 3];
indivPairs = [2, 6];%[0, 2; 0, 4; 0, 6; 0, 1; 0, 4; 1, 4; 2, 4; 4, 6];
%indivPairs = [0, 4; 0, 9; 4, 9];

% Sum up the likelihoods for the first totalChr chromosomes.
totalChr = 22;

rSize = length(inRels);
if doTree == 1,
    rSize = size(indivPairs, 1);
end

for r = 1:rSize,
    subdir = '';
    pairType = '';
    if doTree == 1,
        if doLevels == 1,
           subdir = strcat('indiv_l', int2str(levelPairs(r, 1)), '_i', ...
               int2str(indivPairs(r, 1)), '_indiv_l', ...
               int2str(levelPairs(r, 2)), '_i', int2str(indivPairs(r, 2))); 
        else
            subdir = strcat('indiv', int2str(indivPairs(r, 1)), '_indiv', int2str(indivPairs(r, 2)));
        end
    else
        pairType = strcat('Pair_', inRels{r});
    end
    for i = minIndiv:maxIndiv,
        pair = '';
        if doTree == 1,
            if doLevels == 1,
                pair = strcat('Tree_t1_', int2str(i));
            else
                pair = strcat('Tree_2_4_4_', int2str(i));
            end
        else
            pair = strcat(setPref, '_', pairType, '_', int2str(i));
        end

        % A single line of data, one col per relationship.
        likelihoods = zeros(1, numCols);
        for c = 1:totalChr,
            % Get the likelihoods for the chromosome.
            chrLikelihoods = zeros(1, numCols);
            % Read all the specified input files. 
            for j = 1:length(suf),
                if doTree == 1,
                    if doLevels == 1,
                        filename = strcat(likePref, setPref, '/Trees/Tree_t1_L/', ...
                            pair, '/', subdir, '/', pair, '_', subdir, ...
                            '_chr', int2str(c), '_pruned', suf{j}, '.logProb');
                    else
                        filename = strcat(likePref, setPref, '/Trees/Tree_2_4_4/', ...
                            '/', pair, '/', subdir, '/', pair, '_indiv_', int2str(indivPairs(r, 1)), ...
                            '_indiv_', int2str(indivPairs(r, 2)), ...
                            '_chr', int2str(c), '_pruned', suf{j}, '.logProb');
                    end
                else
                    filename = strcat(likePref, setPref, '/', pairType, '/', pair, ...
                        '/', pair, '_chr', int2str(c), '_pruned', suf{j}, ...
                        '.logProb');
                end
                
                temp = importdata(filename, '\t', 3);
                % For each relationship in the input file, check if it matches
                % one of the relationships in the output.
                for h = 1:length(temp.colheaders),
                    head = strrep(temp.colheaders{h}, '#', '');
                    for col = 1:numCols,
                        if strcmp(outRels{col}, head),
                            chrLikelihoods(col) = temp.data(1, h);
                            break;
                        end
                    end
                end
            end
            likelihoods = likelihoods + chrLikelihoods;
        end
        
        if doTree == 1,
            if doLevels == 1,
                outFilename = strcat(likePref, setPref, '/Trees/Tree_t1_L/', ...
                    pair, '/', subdir, '/', pair, '_', subdir, outSuf, '.logProb')
            else
                outFilename = strcat(likePref, setPref, '/Trees/Tree_2_4_4/', ...
                        pair, '/', subdir, '/', pair, '_indiv_', int2str(indivPairs(r, 1)), ...
                        '_indiv_', int2str(indivPairs(r, 2)), ...
                        outSuf, '.logProb');
            end
        else
            outFilename = strcat(likePref, setPref, '/', pairType, '/', pair, ...
                '/', pair, outSuf, '.logProb');
        end
        outfile = fopen(outFilename, 'w');
        fprintf(outfile, '%s\n', temp.textdata{1});
        fprintf(outfile, '%s\n', temp.textdata{2});
        fprintf(outfile, '#');
        for j = 1:(numCols - 1),
            fprintf(outfile, '%s\t', outRels{j});
        end
        fprintf(outfile, '%s\n', outRels{numCols});
        fclose(outfile);
        dlmwrite(outFilename, likelihoods, '-append', 'delimiter', '\t', 'newline', 'unix', 'precision', '%.2f');
    end
end