% Reads a postcounts file and returns the haplocount features and the
% percentage of IBD.
function [counts, blockStats] = computePostCounts(temp, startCol, endCol, ...
    startSizeCol, endSizeCol, sizeCutoffs, chromSizes, chromNumSnps, ...
    freqAcol, freqBcol, chrCol)

totalSNPs = sum(chromNumSnps);
counts = zeros(1, 4);
blockStats = 0.0;
selectors = zeros(size(temp, 1), 1);
blocks = zeros(100, 4);
numBlocks = 0;
for c = 1:22,
    % Get the transitions for the chromosome.
    chromData = temp(temp(:, chrCol) == c, :);
    [chromSel, chromBlocks] = trans2Blocks(chromData, startCol, endCol, ...
        startSizeCol, endSizeCol,...
        sizeCutoffs(c), chromSizes(c), chromNumSnps(c));
    blocks((numBlocks + 1):(numBlocks + size(chromBlocks, 1)), :) = chromBlocks;
    numBlocks = numBlocks + size(chromBlocks, 1);
    selectors(temp(:, chrCol) == c) = chromSel;
end
%find(selectors == 0)
sel = temp(selectors == 1, :);
if ~isempty(sel),
    % mean log ratio of freqA and freqB over all transitions
    counts(1) = mean(sel(:, freqAcol) - sel(:, freqBcol));
    counts(2) = sum(sel(:, freqAcol) - sel(:, freqBcol) < 0) / size(sel, 1);
    counts(3) = sum(sel(:, freqAcol) - sel(:, freqBcol) > 0) / size(sel, 1);
end
% number of transitions.
counts(4) = size(sel, 1);
ibdBlockIdx = find(blocks(1:numBlocks, 1));
if ~isempty(ibdBlockIdx),
    blockStats = sum(blocks(ibdBlockIdx, 3) - blocks(ibdBlockIdx, 2)) / totalSNPs;
end
end