% rels: the trueRels that will be in the returned **inverted data**. The
% corresponding inverted rel must be in **data**.
% trueRels: the trueRels of the first dimension of data
% assumedRels: the assumedRels of data
function inverseData = invertData(data, rels, trueRels, assumedRels)
% Remap the assumed rels, so that you can remap the features.
% For each assumed rel, get the inverse and check if it is in the assumed
% rels. For example, the features of 2_0_3 will be replaced with the
% features of 2_3_0 and vice versa. So if i is the index of 2_0_3 in
% assumedRels and j is the index of 2_3_0, inverseAssumedRelIdx(i) = j and
% inverseAssumedRelIdx(j) = i. If k is the index of 2_1_1, then
% inverseAssumedRelIdx(k) = k. So basically at this point we figure out how
% we need to shuffle the columns/features.
inverseAssumedRels = getInverseRels(assumedRels);
inverseAssumedRelIdx = 1:length(assumedRels);
for r = 1:length(assumedRels),
    found = 0;
    for r1 = 1:length(inverseAssumedRels),
        if strcmp(inverseAssumedRels{r1}, assumedRels{r}),
            inverseAssumedRelIdx(r) = r1;
            found = 1;
            break;
        end
    end
    if ~found,
        warning('Inverse of assumed rel %s not found, using %s instead', ...
            assumedRels{r}, assumedRels{inverseAssumedRelIdx(r)});
    end
end
inverseAssumedRels
inverseAssumedRelIdx

% Now find which true relationships will be used to get the "inverted"
% rels.
inverseRels = getInverseRels(rels);
inverseRelIdx = 1:length(rels);
for r = 1:length(inverseRels),
    found = 0;
    for r1 = 1:length(trueRels),
        if strcmp(inverseRels{r}, trueRels{r1}),
            inverseRelIdx(r) = r1;
            found = 1;
            break;
        end
    end
    if ~found,
        err = MException('', 'Inverse rel %s not found in true rels', inverseRels{r});
        throw(err);
    end
end

numCounts = 4; % haplocount features
numBlockStats = 1; % block statistics
numCols = numCounts * length(assumedRels) + numBlockStats * length(assumedRels) + length(assumedRels)
blockStartIdx = numCounts * length(assumedRels) + 1;
likeStartIdx = blockStartIdx + numBlockStats * length(assumedRels);


% Map the indices of the features in data to the indices of the features in
% inverseData.
newAssumedRelIdx = [];
% The first haplo-count for each assumed rel will be negated.
negateIdx = 1:numCounts:(blockStartIdx - 1)
% The second and third haplo-counts for each assumed rel will be swapped.
haplo1idx = 2:numCounts:(blockStartIdx - 1)
haplo2idx = 3:numCounts:(blockStartIdx - 1)
for r = 1:length(assumedRels),
    newAssumedRelIdx = [newAssumedRelIdx, (inverseAssumedRelIdx(r) - 1) * numCounts + [1:numCounts]];
end
for r = 1:length(assumedRels),
    newAssumedRelIdx = [newAssumedRelIdx, blockStartIdx - 1 + (inverseAssumedRelIdx(r) - 1) * numBlockStats + [1:numBlockStats]];
end
for r = 1:length(assumedRels),
    newAssumedRelIdx = [newAssumedRelIdx, likeStartIdx + inverseAssumedRelIdx(r) - 1];
end
newAssumedRelIdx

inverseData = zeros(length(rels), size(data, 2), size(data, 3));
for r = 1:length(rels),
    inverseData(r, :, :) = data(inverseRelIdx(r), :, newAssumedRelIdx);
    inverseData(r, :, negateIdx) = -inverseData(r, :, negateIdx);
    temp = inverseData(r, :, haplo1idx);
    inverseData(r, :, haplo1idx) = inverseData(r, :, haplo2idx);
    inverseData(r, :, haplo2idx) = temp;
end
end
    