function data = loadData(pairPref, countsPref, countsSuf, likePref, likeSuf, ...
        cutoffFile, ibdPref, ibdSuf, sizesPath, trueRels, assumedRels, sets, ...
        minIndiv, maxIndiv, percent, hasCounts)
    
% Cutoffs for filtering out small IBD blocks.
%cutoffs = loadCutoffs(distPref, assumedRels, percent);
readCutoffs = load(cutoffFile);
if readCutoffs.percent ~= percent,
    err = MException('Incompatible cutoff file and cutoff percentage');
    throw(err);
end
cutoffs = selectGenetDistCutoffs(assumedRels, readCutoffs.cutoffs, readCutoffs.cutoffRels);

chrSizes = importdata(sizesPath, '\t', 1);
% Number of SNPs per chromosome.
chrSNPsizes = chrSizes.data(:, 3) - chrSizes.data(:, 2) + 1;
% Genetic position of the last SNP of each chromosome.
chrGenetSizes = chrSizes.data(:, 7);

numCounts = 4; % haplocount features
numBlockStats = 1; % block statistics
numCols = numCounts * length(assumedRels) + numBlockStats * length(assumedRels) + length(assumedRels);
blockStartIdx = numCounts * length(assumedRels) + 1;
likeStartIdx = blockStartIdx + numBlockStats * length(assumedRels);

% Individuals per set per relationship
numIndivSet = maxIndiv - minIndiv + 1;
% Total number of individuals (training + testing) per relationship
numIndiv = (maxIndiv - minIndiv + 1) * length(sets);

% data(r, i, c) is feature c for pair i and (true) relationship r.
data = zeros(length(trueRels), numIndiv, numCols);

% Read the haplocounts and likelihoods for the individuals of each
% relationship and permute them.
for r = 1:length(trueRels),
    % Pair_X_X_X
    pairType = strcat('Pair_', trueRels{r});
    for s = 1:length(sets),
        % like CEU_TSI_Set_X
        setPref = strcat(pairPref, '_Set_', int2str(sets(s)));
        % like CEU_TSI_Set_X_Pair_X_X_X_
        pairSetPref = strcat(setPref, '_', pairType, '_');
        %perm = randperm(numIndivSet);
        % Indices for the individuals of the set
        indivIndices = ((s - 1) * numIndivSet + 1):(s * numIndivSet);
        % For each assumed relationship...
        for r2 = 1:length(assumedRels),
            countIndices = ((r2 - 1) * numCounts + 1):(r2 * numCounts);
            blockIndices = ((r2 - 1) * numBlockStats + blockStartIdx):(r2 * numBlockStats + blockStartIdx - 1);    
            if hasCounts && ~strcmp(assumedRels{r2}, 'unrel'),
                [counts, blockStats] = loadPostCounts(minIndiv, maxIndiv, ...
                    strcat(countsPref, setPref, '/', pairType), pairSetPref, ...
                    strcat(countsSuf, '_', assumedRels{r2}), ...
                    cutoffs(r2, :), chrGenetSizes, chrSNPsizes);
                data(r, indivIndices, countIndices) = counts(:, :);
                data(r, indivIndices, blockIndices) = loadAvgIbd(...
                    minIndiv, maxIndiv, ...
                    strcat(ibdPref, setPref, '/', pairType), ...
                    strcat(pairSetPref, ibdSuf), assumedRels{r2});
                %data(r, indivIndices, blockIndices) = blockStats(:, :);
            elseif hasCounts && strcmp(assumedRels{r2}, 'unrel'),
                data(r, indivIndices, countIndices) = 0.0;
                data(r, indivIndices, blockIndices) = 0.0;
            else
                data(r, indivIndices, countIndices) = -1;
                data(r, indivIndices, blockIndices) = -1;
            end
        end
        
        % Now read the likelihoods.
        likelihoods = loadLikelihoods(minIndiv, maxIndiv, ...
            strcat(likePref, setPref, '/', pairType), pairSetPref, ...
            likeSuf, assumedRels);
        % Divide with maximum likelihood
        %     for c = 1:size(likelihoods, 1),
        %         likelihoods(c, :) = likelihoods(c, :) - max(likelihoods(c, :));
        %     end
        data(r, indivIndices, likeStartIdx:numCols) = likelihoods(:, :);
    end
end

% classifier can't handle NaNs and Infs. Maybe 0 is not the best option.
%data(isnan(data)) = 0.0;
%data(isinf(data)) = 0.0;
end