% Read the likelihoods for all individuals from minIndiv to maxIndiv.
% The likelihoods for individual i are stored in
% <likePref>/<pairPref><i>/<pairPref><i><likeSuf>.logProb.
% The likelihood file must have the likelihoods for all relationships in
% postRels (which must be a cell array).
% Return value:
% likelihoods(i, j) will be the likelhood for individual i and relationship
% postRels{j}.
function likelihoods = loadLikelihoods(minIndiv, maxIndiv, ...
    likePref, pairPref, likeSuf, postRels, subdir)

if nargin == 6,
    subdir = '';
else
    subdir = strcat(subdir, '/');
end

numIndiv = maxIndiv - minIndiv + 1;
likelihoods = zeros(numIndiv, length(postRels));
% The likelihoods in the file are not necessarily in the same order as in
% the rells array, and the file might have more relationships.
% relIndices(r) will be the column of the likelihood file that corresponds
% to postRels{r}.
relIndices = zeros(1, length(postRels));

for i = minIndiv:maxIndiv,
    likeFile = strcat(likePref, '/', pairPref, num2str(i), ...
        '/', subdir, pairPref, num2str(i), likeSuf, '.logProb')
    % Assume 3 lines of comments in the likelihoods file.
    temp = importdata(likeFile, '\t', 3);
    for r = 1:length(postRels),
        for h = 1:length(temp.colheaders),
            head = strrep(temp.colheaders{h}, '#', '');
            if strcmp(postRels{r}, head),
                relIndices(r) = h;
                break;
            end
        end
    end
    % If some of the relationships in postRels are not in the file, then
    % then some relIndices will be 0 and this will break.
    likelihoods(i - minIndiv + 1, :) = temp.data(1, relIndices);
end
end