% Reads the postcounts for all individuals from minIndiv to maxIndiv.
% The input files are assumed to be
% countPref/pairPref<i>/pairPref<i>suf
% 
% chromNumSnps: vector with the number of snps per chromosome
function [counts, blockStats] = loadPostCounts(minIndiv, maxIndiv, ...
        countPref, pairPref, suf, sizeCutoffs, chromSizes, chromNumSnps, ...
        subdir)
   
    if nargin == 8,
        subdir = '';
    else
        subdir = strcat(subdir, '/');
    end
    numIndiv = maxIndiv - minIndiv + 1;
    
    chrCol = 4;
    startCol = 5;
    endCol = 6;
    %transPosCol = 6;
    startSizeCol = 11;
    %transSizeCol = 12;
    endSizeCol = 12;
    freqAcol = 13;
    freqBcol = 14;
    
    counts = zeros(numIndiv, 4);
    blockStats = zeros(numIndiv, 1);
    
    for i = minIndiv:maxIndiv,
        countFilePref = strcat(countPref, '/', pairPref, num2str(i), '/', ...
            subdir, pairPref, num2str(i), suf);
        countFile = strcat(countFilePref, '.postcounts2');
        if exist(countFile, 'file') ~= 2,
            countFile = strcat(countFilePref, '.postcounts');
            freqAcol = 15;
            freqBcol = 16;
        end
        tempStruct = importdata(countFile, '\t', 1);
        try
            temp = tempStruct.data;
            [haploCounts, stats] = computePostCounts(temp, startCol, endCol, startSizeCol,...
                endSizeCol, sizeCutoffs, chromSizes, chromNumSnps, ...
                freqAcol, freqBcol, chrCol);
            idx = i - minIndiv + 1;
            counts(idx, :) = haploCounts;
            blockStats(idx, 1) = stats;
        catch ME
            warning('MATLAB:emptyFile', 'Empty file %s', countFile)
        end
    end
    
%     for i = minIndiv:maxIndiv,
%         countFile = strcat(countPref, trueRel, '/', pairPref, ...
%             '_Pair_', trueRel, '_', num2str(i), '/', pairPref, '_Pair_', trueRel, ...
%             '_', num2str(i), '_w20', ...
%             '_i', num2str(expIbdCutoff), '_n', num2str(expNonIbdCutoff), ...
%             '_c', num2str(expCutoff), '_', assumedRel, '.postcounts');
%         tempStruct = importdata(countFile, '\t', 1);
%         temp = tempStruct.data;
%         chroms = temp(:, chrCol);
%         selectors = zeros(size(temp, 1), 1);
%         for j = 1:size(temp, 1),
%             % The following code might not filter correctly blocks at the 
%             % end of the chromosome.
%             % If this is a transition 1->0, then startSizeCol is the start of 
%             % the IBD block and endSizeCol is the end of the IBD block.
%             if temp(j, 1) == 1 && temp(j, 2) == 0 && ...
%                 temp(j, endSizeCol) - temp(j, startSizeCol) > sizeCutoffs(chroms(j)),
%                 selectors(j) = 1;
%                 %blocks(j, :) = [1, temp(j, startSizeCol), temp(j, endSizeCol)]; 
%             % If this is a transition 0->1, then the following transition
%             % is probably 1->0, so we look at the following transition to
%             % see if the block passed the cutoff. If the following
%             % transition is not a 1->0 transition on the same chromosome,
%             % we just pretend that the block passed the cutoff.
%             elseif (temp(j, 1) == 0 && j == size(temp, 1)) || ...
%                     (temp(j, 1) == 0 && temp(j, chrCol) ~= temp(j + 1, chrCol)) || ...
%                     (temp(j, 1) == 0 && temp(j + 1, 1) == 1 && ...
%                     temp(j, chrCol) == temp(j + 1, chrCol) && ...
%                     temp(j, endSizeCol) == temp(j + 1, startSizeCol) && ...
%                     temp(j + 1, endSizeCol) - temp(j + 1, startSizeCol) > sizeCutoffs(chroms(j)))
%                 selectors(j) = 1;
%                 %blocks(j, :) = [0, temp
%             end
%         end
%         %i
%         %find(selectors == 0)
%         sel = temp(selectors == 1, :);
%         idx = i - minIndiv + 1;
%         if ~isempty(sel),
%             % mean log ratio of freqA and freqB over all transitions
%             counts(idx, 1) = mean(sel(:, freqAcol) - sel(:, freqBcol));
%             counts(idx, 2) = sum(sel(:, freqAcol) - sel(:, freqBcol) < 0) / size(sel, 1);
%             counts(idx, 3) = sum(sel(:, freqAcol) - sel(:, freqBcol) > 0) / size(sel, 1);
%         else
%             counts(idx, 1:3) = 0.0;
%         end
%         % number of transitions.
%         counts(idx, 4) = size(sel, 1);
%     end    
end