% inputs : 
%rel1_s : AB relationship in this format : MRCAS_genA_genB
%rel2_s : BC relationship in this format : MRCAs_genB_genC
% Assumption : A and C are relatives. 
% output:
% All possible relationships for AC
function rels = possibleRels(rel1_s,rel2_s)
rel1_s=strrep(rel1_s,'n1','-1');
rel2_s=strrep(rel2_s,'n1','-1');
rel1_decode=textscan(rel1_s,'%s %s %s', 'delimiter', '_');
rel2_decode=textscan(rel2_s,'%s %s %s', 'delimiter', '_');
rel1 = [0,0,0]; rel2 = [0,0,0];
for k=1:3
    rel1(k)=str2double(char(rel1_decode{k}));
    rel2(k)=str2double(char(rel2_decode{k}));
end

if rel2(2) < rel1(3) % C is somewhere in the left subtree of AB pedigree. AC will have the same MRCAs as AB and genA in AC is the same as genA in AB  
    relation = [rel1(1), rel1(2),rel1(3)-rel2(2)+rel2(3)];
elseif rel2(2) == rel1(3)
    if rel1(1) == 2
         if rel2(1)==2
            relation(1,:) = [2, rel1(2), rel2(3)];
            a = rel1(2); b = rel2(3);
            if a || b
                a = a-1; b = b-1;
                while (a>=0) && (b>=0)
                    relation(size(relation,1)+1,:) = [2, a, b];                
                    relation(size(relation,1)+1,:) = [1, a, b];
                    a = a - 1; b = b-1;
                end
                if a ~= b
                    relation(size(relation,1)+1,:) = [1,a,b];
                end

            end
        elseif rel2(1)==1 % C is a descender of another partner of A&B MRCAs 
            relation = [1, rel1(2), rel2(3)];
        end
    
    elseif rel1(1) == 1
        if rel2(1) == 2
            relation = [1, rel1(2), rel2(3)]; 
        else
            relation(1,:) = [1,rel1(2), rel2(3)]; % C is from the third partner of the MRCA
            relation(2,:) = [2,rel1(2), rel2(3)];
            
             a = rel1(2); b = rel2(3);
            if a || b
                a = a-1; b = b-1;
                while (a>=0) && (b>=0)
                    relation(size(relation,1)+1,:) = [2, a, b];                
                    relation(size(relation,1)+1,:) = [1, a, b];
                    a = a - 1; b = b-1;
                end
                if a ~= b
                    relation(size(relation,1)+1,:) = [1,a,b];
                end

            end
                                   
        end
    end
       
else
   if rel2(1)==2
        relation = [2, rel2(2)-rel1(3)+rel1(2), rel2(3)] ;
    elseif rel2(1)==1
        relation = [1, rel2(2)-rel1(3)+rel1(2) , rel2(3)] ;
   end
end

for i=1:size(relation,1)
    rels{i}=strcat(num2str(relation(i,1)),'_',num2str(relation(i,2)),'_',num2str(relation(i,3)));
    rels{i} = strrep(rels{i},'-1','n1');
end
    