function [dist01, dist10, prec, rec] = postTransMeanError(ibdCut, nonIbdCut, win, percent, diagOnly)
postPref = '../Likelihoods/hapmap3_CEU_TSI_phaseErr002/CEU_TSI_Set_1/';
trueIBDpref = '../TrueIBD/hapmap3_CEU_TSI_phaseErr002/CEU_TSI_Set_1/';
pairPref = 'CEU_TSI_Set_1';
mapPref = '../../hapmap/hapmap3/CEU_TSI/CEU_TSI_';
distPref = '../BlockGeneticDist/hapmap3_CEU/Complete/Pair_';

minIndiv = 1;
maxIndiv = 1;
minChr = 1;
maxChr = 22;
numIndiv = maxIndiv - minIndiv + 1;
numChr = maxChr - minChr + 1;
maxTrans = 40 * numIndiv;

styles = {'-b', '-r', '-g', '-m', '-c', '--b', '--r', '--g', '--m', '--c'};

rels3 = {'2_0_3', '2_1_2', '1_n1_3', '1_0_2', '1_1_1', '2_1_1', '2_2_2'};
rels4 = {'2_1_2', '1_n1_3', '1_0_2', '1_1_1', '2_1_1', '2_2_2'};

trueRels = {'2_1_2'};
postRels = rels3;

winLen = 2 * win;

trueRelTitles = trueRels;
postRelTitles = postRels;
for r = 1:length(trueRels),
    trueRelTitles{r} = strrep(trueRels{r}, '_', ':');
end
for r = 1:length(postRels),
    postRelTitles{r} = strrep(postRels{r}, '_', ':');
end

cutoffs = zeros(length(postRels), 22);
for r = 1:length(postRels),
    for c = 1:22,
        dists = importdata(strcat(distPref, postRels{r}, '/Pair_', postRels{r}, ...
            '_chr', int2str(c), '_Merged_10000'));
        cutoffs(r, c) = prctile(dists, percent);
    end
end
    
totalTrans = zeros(length(trueRels), 1);
transCounts = zeros(length(trueRels), length(postRels), 3);
totalBlocks = zeros(length(trueRels), length(postRels));
blockCounts = zeros(length(trueRels), length(postRels), 3);
overlapDists01 = zeros(40 * length(trueRels) * numIndiv, 1);
added01 = 0;
overlapDists10 = zeros(40 * length(trueRels) * numIndiv, 1);
added10 = 0;
    
for r1 =  1:length(trueRels),
    pairType = strcat('Pair_', trueRels{r1});
    for i = minIndiv:maxIndiv,
        pairName = strcat(pairPref, '_', pairType, '_', int2str(i));
        for c = minChr:maxChr,
            chrName = strcat('chr', int2str(c));
            % Read the true IBD for the unpruned chromosome
            trueIBDtemp = importdata(strcat(trueIBDpref, pairType, '/', pairName, '/', pairName, '_', chrName,'.trueibd'), '\t');
            trueIBD = trueIBDtemp(:, 1);
            numSNPs = length(trueIBD);
            % Get the transitions 1->0 and 0->1. If trueIBD(i) = 1 and
            % trueIBD(i+1) = 0, then there is a transition between i
            % and i+1 and i will be in trans10.
            trans01 = find(~trueIBD(1:(numSNPs - 1)) & trueIBD(2:numSNPs));
            trans10 = find(trueIBD(1:(numSNPs - 1)) & ~trueIBD(2:numSNPs));
            totalTrans(r1) = totalTrans(r1) + length(trans01) + length(trans10);
            % Read map from full data to pruned data.
            map = importdata(strcat(mapPref, 'chrom', int2str(c), '.map'), '\t');
            % Convert from 0-based to  1-based.
            map(:, 1) = map(:, 1) + 1;
            mappedPos = find(map(:, 2) > -1);
            map(mappedPos, 2) = map(mappedPos, 2) + 1;
            % Create map from pruned data to full data.
            prunedMap = map(mappedPos, [2,1]);
            for r2 = 1:length(postRels),
                if diagOnly == 1 && ~strcmp(trueRels{r1}, postRels{r2}),
                    continue;
                end
                % Read the posteriors for that chromosome.
                postIBDtemp = importdata(strcat(postPref, pairType, '/', pairName, '/', pairName, '_', chrName, '_pruned_', postRels{r2}, '.postibd'), '\t');
                postIBD = exp(postIBDtemp(:, 1));
                blocks = post2Blocks(postIBD, map, prunedMap, 4, cutoffs(r2, c), ibdCut, nonIbdCut);
                transBlocksTemp01 = ones(size(blocks, 1), 2) * (-2);
                transBlocksTemp10 = ones(size(blocks, 1), 2) * (-2);
                numTrans01 = 0;
                numTrans10 = 0;
                for j = 1:size(blocks, 1),
                    if blocks(j, 1) == -1 && j > 1 && j < size(blocks, 1),
                        if blocks(j - 1, 1) == 0 && blocks(j + 1, 1) == 1,
                            numTrans01 = numTrans01 + 1;
                            % Extend the blocks to the left and to the
                            % right by 1 SNP before mapping to full data.
                            transBlocksTemp01(numTrans01, :) = [blocks(j, 2) - 1, blocks(j, 3) + 1];
                        elseif blocks(j - 1, 1) == 1 && blocks(j + 1, 1) == 0,
                            numTrans10 = numTrans10 + 1;
                            transBlocksTemp10(numTrans10, :) = [blocks(j, 2) - 1, blocks(j, 3) + 1];
                        end
                    end
                end
                transBlocks01 = [prunedMap(transBlocksTemp01(1:numTrans01, 1), 2), prunedMap(transBlocksTemp01(1:numTrans01, 2), 2)];
                transBlocks01(:, 1) = transBlocks01(:, 1) - win;
                transBlocks01(:, 2) = transBlocks01(:, 2) + win;

                transBlocks10 = [prunedMap(transBlocksTemp10(1:numTrans10, 1), 2), prunedMap(transBlocksTemp10(1:numTrans10, 2), 2)];
                transBlocks10(:, 1) = transBlocks10(:, 1) - win;
                transBlocks10(:, 2) = transBlocks10(:, 2) + win;

                totalBlocks(r1, r2) = totalBlocks(r1, r2) + size(transBlocks01, 1) + size(transBlocks10, 1);
                
                [transOverlaps01, blockOverlaps01] = transOverlapCountsAndDist(trans01, transBlocks01);
                [transOverlaps10, blockOverlaps10] = transOverlapCountsAndDist(trans10, transBlocks10);
                newCounts = [sum(transOverlaps01(:, 1) == 0) + sum(transOverlaps10(:, 1) == 0), ...
                    sum(transOverlaps01(:, 1) == 1) + sum(transOverlaps10(:, 1) == 1), ...
                    sum(transOverlaps01(:, 1) > 1) + sum(transOverlaps10(:, 1) > 1)];
                for j = 1:3,
                    transCounts(r1, r2, j) = transCounts(r1, r2, j) + newCounts(j);
                end
                newCounts = [sum(blockOverlaps01(:, 1) == 0) + sum(blockOverlaps10(:, 1) == 0), ...
                    sum(blockOverlaps01(:, 1) == 1) + sum(blockOverlaps10(:, 1) == 1), ...
                    sum(blockOverlaps01(:, 1) > 1) + sum(blockOverlaps10(:, 1) > 1)];
                for j = 1:3,
                    blockCounts(r1, r2, j) = blockCounts(r1, r2, j) + newCounts(j);
                end

                if strcmp(trueRels{r1}, postRels{r2}),
                    addIdx = find(transOverlaps01(:, 1));
                    overlapDists01((added01 + 1):(added01 + length(addIdx))) = transOverlaps01(addIdx, 2);
                    added01 = added01 + length(addIdx);
                    addIdx = find(transOverlaps10(:, 1));
                    overlapDists10((added10 + 1):(added10 + length(addIdx))) = transOverlaps10(addIdx, 2);
                    added10 = added10 + length(addIdx);
                end
            end
        end
    end
end

% figure();
% hist(overlapDists01(1:added01));
% figure();
% hist(overlapDists10(1:added10));

dist01 = mean(overlapDists01(1:added01));
dist10 = mean(overlapDists10(1:added10));
rec = sum(diag(transCounts(:, :, 2))) / sum(totalTrans);
prec = sum(diag(blockCounts(:, :, 2))) / sum(diag(totalBlocks));
fprintf('i = %.2f, n = %.2f, w = %d, p = %.2f: %.6f\t%.6f\t%.6f\t%.6f\n', ...
                ibdCut, nonIbdCut, win, percent, dist01, dist10, prec, rec);
% 
% fprintf('True trans counts\n');
% for i = 1:length(trueRels),
%     transCounts(i, :, :) = transCounts(i, :, :) / totalTrans(i);
% end
% disp(postRels);
% disp(transCounts);
% fprintf('Post block counts\n');
% for i = 1:size(blockCounts, 3),
%     blockCounts(:, :, i) = blockCounts(:, :, i) ./ totalBlocks;
% end
% disp(postRels);
% disp(blockCounts);
end
