clear all

countsPref = '../PostCounts/hapmap3_CEU_TSI_Train_vs_CEU_TSI_Test/';
likePref = '../Likelihoods/hapmap3_CEU_TSI_Train_vs_CEU_TSI_Test/';
trueIBDpref = '../TrueIBD/hapmap3_CEU_TSI_Train/';
pairPref = 'CEU_TSI';
sizesPath = '../../hapmap/hapmap3/HAPAA/CEU_TSI/CEU_TSI_pruned.chrSizes';
countsSuf = '_w20_i0.9_n0.6_c0.5_';

minIndiv = 1;
maxIndiv = 10;
minChr = 1;
maxChr = 22;
numIndiv = maxIndiv - minIndiv + 1;
numChr = maxChr - minChr + 1;

rels3 = {'2_0_3', '2_1_2', '1_n1_3', '1_0_2', '1_1_1', '2_1_1', '2_2_2'};
rels4 = {'2_1_2'};
trueRels = rels3;
postRels = rels3;

chrCol = 4;
startCol = 5;
endCol = 6;

chrSizes = importdata(sizesPath, '\t', 1);
chrSNPsizes = chrSizes.data(:, 3) - chrSizes.data(:, 2) + 1;

tp = zeros(length(trueRels), length(postRels));
fp = zeros(length(trueRels), length(postRels));
tn = zeros(length(trueRels), length(postRels));
fn = zeros(length(trueRels), length(postRels));
trueBlockCounts = zeros(length(trueRels), length(postRels), 3);
numTrueBlocks = zeros(length(trueRels), 1);
postBlockCounts = zeros(length(trueRels), length(postRels), 3);
numPostBlocks = zeros(length(trueRels), length(postRels));

for r1 =  1:length(trueRels),
    pairType = strcat('Pair_', trueRels{r1});
    for i = minIndiv:maxIndiv,
        pairName = strcat(pairPref, '_', pairType, '_', int2str(i));
        for r2 = 1:length(postRels),
            % Read the posterior transitions for the individual (for all
            % chromosomes).
            postCounts = importdata(strcat(countsPref, pairType, '/', pairName, '/', pairName, countsSuf, postRels{r2}, '.postcounts'), '\t', 1);
            for c = minChr:maxChr,
                chrName = strcat('chr', int2str(c));
                % Read the true IBD for the chromosome
                trueIBDtemp = importdata(strcat(trueIBDpref, pairType, '/', pairName, '/', pairName, '_', chrName, '_pruned.trueibd'), '\t');
                trueIBD = trueIBDtemp(:, 1);
                % Restrict postCounts to this chromosome.
                countsData = postCounts.data(postCounts.data(:, chrCol) == c, :);
                % countIBD will be the IBD vector based on the transitions in
                % postcounts
                countIBD = zeros(size(trueIBD, 1), 1);
                % index of last transition from non-IBD to IBD
                last01 = 0;
                % index of last transition from IBD to non-IBD
                last10 = 0;
                % If there is no transition, then the whole chromosome is
                % assumed to be non-IBD (there is a small chance that the
                % whole chromosome is IBD, but let's ignore that.)
                if ~isempty(countsData)
                    % First transition is 1->0, so we call IBD all the SNPs
                    % up to the transition. 
                    if countsData(1, 1) > 0,
                        % endCol has the (0-based) position right after the
                        % transition, that is, the first position of the
                        % non-IBD block. Therefore, endCol has the 1-based 
                        % end of the IBD block.
                        countIBD(1:countsData(1, endCol)) = 1;
                        last10 = 1;
                    else
                        last01 = 1;
                    end
                    for j = 2:size(countsData, 1),
                        % This will never be the case, since we're
                        % selecing the transitions of a single
                        % chromosome...
                        if countsData(j - 1, chrCol) ~= countsData(j, chrCol),
                            if last01 > last10,
                                blockStart = countsData(last01, endCol) + 1;
                                blockEnd = chrSNPsizes(countsData(last01, chrCol));
                                countIBD(blockStart:blockEnd) = 1;
                            end
                            last01 = 0;
                            last10 = 0;
                        end
                        if countsData(j, 1) == 0 && countsData(j, 2) > 0,
                            last01 = j;
                        elseif countsData(j, 1) > 0 && countsData(j, 2) == 0,
                            blockStart = countsData(j, startCol) + 1;
                            if last01 > 0 && last01 < j - 1 && ...
                                    countsData(last01, chrCol) == countsData(j, chrCol),
                                blockStart = countsData(last01, endCol) + 1;
                            end
                            countIBD(blockStart:countsData(j, endCol)) = 1;
                        end
                    end
                    countsSize = size(countsData, 1);
                    % Last transition is to IBD, so we call IBD all the SNPs from the
                    % transition to the last SNP.
                    if countsData(countsSize, 2) > 0,
                        blockEnd = chrSNPsizes(countsData(countsSize, chrCol));
                        countIBD((countsData(countsSize, endCol) + 1):blockEnd) = 1;
                    end
                end               
                % Find positions that are 1 in countIBD and 0 in trueIBD
                postIBDfp = find(countIBD & ~trueIBD);
                fp(r1, r2) = fp(r1, r2) + length(postIBDfp);
                postIBDfn = find(trueIBD & ~countIBD);
                fn(r1, r2) = fn(r1, r2) + length(postIBDfn);
                tp(r1, r2) = tp(r1, r2) + sum(countIBD & trueIBD);
                tn(r1, r2) = tn(r1, r2) + sum(~countIBD & ~trueIBD);
                
                % Get the blocks from the indicators.
                trueBlocks = indicators2Blocks(trueIBD);
                postBlocks = indicators2Blocks(countIBD);
                
                % numOverlaps(i) will be the number of posterior blocks
                % that overlap with the i-th true block.
                numOverlapsTrue = blockOverlapCounts(trueBlocks, postBlocks);
                numOverlapsPost = blockOverlapCounts(postBlocks, trueBlocks);
                
                % numTrueBlocks(r) will be the number of true blocks in all
                % chromosomes of all pairs from the r-th relationship
                if r2 == 1,
                    % Be careful! I read the same true IBD files once for
                    % each relationship in postRels.
                    numTrueBlocks(r1) = numTrueBlocks(r1) + size(trueBlocks, 1);
                end
                
                % numPostBlocks(r1, r2) will be the total number of
                % posterior blocks when r1 is the true relationship and r2
                % is the assumed relationship.
                numPostBlocks(r1, r2) = numPostBlocks(r1, r2) + size(postBlocks, 1);
                
                % trueBlockCounts(r1, r2, 1) will be the number of true blocks
                % that overlap exactly one posterior block, when the
                % assumed relationship is r2.
                newCounts = [sum(numOverlapsTrue == 0), ...
                    sum(numOverlapsTrue == 1), sum(numOverlapsTrue > 1)];
                for j = 1:3,
                    trueBlockCounts(r1, r2, j) = trueBlockCounts(r1, r2, j) + newCounts(j);
                end
                newCounts = [sum(numOverlapsPost == 0), ...
                    sum(numOverlapsPost == 1), sum(numOverlapsPost > 1)];
                for j = 1:3,
                    postBlockCounts(r1, r2, j) = postBlockCounts(r1, r2, j) + newCounts(j);
                end
            end
        end
    end
end

prec = tp ./ (tp + fp);
rec = tp ./ (tp + fn);
fprintf('IBD call precision\n');
disp(postRels);
disp(prec);
fprintf('IBD call recall\n');
disp(postRels);
disp(rec);
fprintf('True blocks per pair\n');
disp(trueRels);
disp(numTrueBlocks' / numIndiv);
fprintf('Post blocks per pair\n');
disp(postRels);
disp(numPostBlocks / numIndiv);
fprintf('True block counts\n');
for i = 1:length(trueRels),
    trueBlockCounts(i, :, :) = trueBlockCounts(i, :, :) / numTrueBlocks(i);
end
disp(postRels);
disp(trueBlockCounts);
fprintf('Post block counts\n');
for i = 1:size(postBlockCounts, 3),
    postBlockCounts(:, :, i) = postBlockCounts(:, :, i) ./ numPostBlocks;
end
disp(postRels);
disp(postBlockCounts);