% Returns the cutoffs for the genetic distances for the relationships in
% rels.  cutoffs(r, c) is the cutoff for cutoffRels{r} and chromosome c.
% selectedCutoffs(r, c) will be the cutoff for rels{r} and chromosome c.
function selectedCutoffs = selectGenetDistCutoffs(rels, cutoffs, cutoffRels)
inverseRels = getInverseRels(rels);
degrees = getRelDegree(rels);
cutoffRelDegrees = getRelDegree(cutoffRels);
selectedCutoffs = zeros(length(rels), size(cutoffs, 2));

for r = 1:length(rels),
    found = 0;
    if strcmp(rels{r}, 'unrel'),
        selectedCutoffs(r, :) = zeros(1, size(cutoffs, 2));
        continue;
    end
    % If rels{r} is in cutoffRels, use the cutoffs from cutoffRels.
    for r1 = 1:length(cutoffRels),
        if strcmp(rels{r}, cutoffRels{r1})
            selectedCutoffs(r, :) = cutoffs(r1, :);
            found = 1;
            break;
        end
    end
    % If rels{r} is not in cutoffRels, try using the cutoffs for the
    % inverse relationship.
    if ~found,
        for r1 = 1:length(cutoffRels),
            if strcmp(inverseRels{r}, cutoffRels{r1}),
                selectedCutoffs(r, :) = cutoffs(r1, :);
                found = 1;
                break;
            end
        end
    end
    % As a last choice, try using the cutoffs for another relationship of
    % the same degree.
    if ~found,
        for r1 = 1:length(cutoffRels),
            if degrees(r) == cutoffRelDegrees(r1),
                selectedCutoffs(r, :) = cutoffs(r1, :);
                found = 1;
                break;
            end
        end
    end
    if ~found,
        err = MException('', ['Could not find cutoffs for rel ', rels{r}]);
        throw(err);
    end
end
end